#!/usr/bin/env python3

import asyncio
import datetime
import functools
import json
import logging
import io, os, sys
import pcpp
import time
import yaml
import websockets

START = time.time()
def time_from_start():
  return int((time.time() - START) * 1000) / 1000
def utc():
  return int(time.time() * 1000) / 1000

def yamlpp_load(path):
  with open(path, 'r') as f:
    data = f.read()     # original input
  p = pcpp.Preprocessor()
  p.parse(data)
  f = io.StringIO()
  p.write(f)
  data_ = f.getvalue()    # preprocessed input
  return yaml.load(data_, Loader=yaml.Loader)

def ready(name):
    return json.dumps({
      "message"   : "ready",
      "challenge" : "773bc1c2cf2df900a2d4f60c58b5ddb7",
      "type"      : name,
      "name"      : name,
      "version"   : "2024-11-21",
      "time"      : time_from_start(),
      "utc"       : utc(),
    })

def parse_conf():

  conf = yamlpp_load(sys.argv[1])

  cell_list = conf.get('cell_list', []) + conf.get('nr_cell_list', [])
  for cell_group in conf.get('cell_groups', []):
    cell_list += cell_group.get('cells', [])
  ip, port = conf.get('com_addr', ':').split(':')

  return {
    'cell_count': len(cell_list),
    'ul_chan_count': sum(map(lambda x: x['n_antenna_ul'], cell_list)),
    'dl_chan_count': sum(map(lambda x: x['n_antenna_dl'], cell_list)),
    'log_filename': conf['log_filename'],
    'com_ip': ip,
    'com_port': port,
  }

async def server(echo, parameters):
    async with websockets.serve(functools.partial(echo, parameters), parameters['com_ip'], parameters['com_port']):
        await asyncio.get_running_loop().create_future()  # run forever

def main(name, echo):
   
  parameters = parse_conf()

  with open(parameters['log_filename'], 'w+') as log:
    log.write("# lte{} mock version 2024-11-21 (x86-64)\n".format(name))
    log.write("# Started on {}\n".format(
      datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")))
    log.write("# {}".format(parameters))

  if parameters['com_ip']:
    asyncio.run(server(echo, parameters))
  while True:
    time.sleep(1)

if __name__ == "__main__":
  main()
